function dy = hopper_ode(t,y,hopper)
% hopper_ode 
% Equations of motion for a bouncing ball, used by dashpod_sim.
%   hopper_ode(T,Y,state_foot,state_p) returns the 
%   derivatives vector for the equations of motion of the dashpod
% It uses the input parameter state_foot to determine which eom function
%   to call, sending to it the state_p variable that tells you if
%	 the valve is on/off.
%   
% From ballode.m's original event-checker documentation:
%   hopper_ode(T,Y,'events') returns a zero-crossing vector VALUE evaluated at
%   (T,Y) and two constant vectors ISTERMINAL and DIRECTION.  By default,
%   the solvers of the ODE Suite do not locate zero-crossings.  However, if
%   the ODE solver property Events is set to 'on' with ODESET, the solver
%   calls the ODE file with the flag 'events'.  The ODE file returns the
%   information that the solver uses to locate zero-crossings of the
%   elements in the VALUE vector.  The VALUE vector may be of any length.
%   It is evaluated at the beginning and end of a step, and if any elements
%   make transitions to, from, or through zero (with the directionality
%   specified in DIRECTION), then the zero-crossing point is located.  The
%   ISTERMINAL vector consists of logical 1's and 0's, enabling you to
%   specify whether or not a zero-crossing of the corresponding VALUE
%   element halts the integration.  The DIRECTION vector enables you to
%   specify a desired directionality, positive (1), negative (-1), and don't
%   care (0) for each VALUE element.
%
% In this case, the events are:
%		out1(1) :	Foot lands on ground (only negative crossings halt
%						the integration)
%		out1(2) :	The body pulls the foot upwards through the spring and
%   					damper (only positive crossings)
%		out1(3) :	The body pulls the foot upwards by reaching the end of 
%   					the piston's stroke (only positive crossings)
%		out1(4) :	The force due to the pressure gets below that needed to
%   					keep the body and foot locked at the end of the stroke
%						(only negative crossings stop the integration)
%		out1(5) :	The beginning or end of the duty cycle
%   
%	 Jorge Cham, 11/24/00 - Basic Functionality
%    Jonathan Karpick, 3/1/06 - Simplified

mass_height = y(1);
mass_vel = y(2);
foot_height = y(3);
foot_vel = y(4);

% Return dy/dt = f(t,y).
if hopper.state_foot==hopper.AIR,
    dy = feval('hopper_air',t,y,hopper);
else %GROUND
    dy = feval('hopper_ground',t,y,hopper);
end
   
end
